#include "StdAfx.h"
#include "CallEvent.h"
#include "ServerSIP.h"
#include "LineList.h"
#include "CallList.h"
#include "QueueList.h"
#include "UserList.h"
#include "DialPlanList.h"
#include "StealthList.h"
#include "ServerData.h"

using namespace System;
using namespace System::Collections;
using namespace System::Text;

#define IN_BOUND_CALL_REJECT		0
#define IN_BOUND_CALL_CONNECT_USER	1
#define IN_BOUND_CALL_PUT_IN_QUEUE	2

cCallEvent::cCallEvent(void)
{
}

void cCallEvent::OnIncomingCallFromLine(cCallData^ objCallData, cLineData^ objLineData, String^ sDialNo)
{
    if(objLineData->m_nCallType == IN_BOUND_CALL_REJECT)
    {
		cServerSIP::m_objServerSIP->CloseCall(objCallData);
	    return;
    }

    if (objLineData->m_nCallType == IN_BOUND_CALL_CONNECT_USER)
    {
	    cServerSIP::m_objServerSIP->DialCallToUser(objCallData, objLineData->m_sCallTo);
	    return;
    }

    if (objLineData->m_nCallType == IN_BOUND_CALL_PUT_IN_QUEUE)
    {
	    cQueueData^ objQueueData = cServerSIP::m_objServerSIP->m_objQueueList->FindQueue(objLineData->m_sCallTo);

	    if(objQueueData == nullptr)
	    {
		    cServerSIP::m_objServerSIP->CloseCall(objCallData);
	    }
	    else
	    {
            objCallData->m_nCallType = CALL_TYPE_QUEUE;
		    OnIncomingCallQueueType(objCallData, objQueueData->m_unQueueId);
	    }
    }
}


void cCallEvent::OnIncomingCallFromUser(cCallData^ objUserCallData, cUserData^ objUserData, String^ sDialNo) 
{
    if(objUserData->m_bStealthListener)
    {
        objUserCallData->m_nCallType = CALL_TYPE_STEALTH;
	    OnIncomingCallStealthType(objUserCallData, objUserData, sDialNo);
    }
    else
    {
        objUserCallData->m_nCallType = CALL_TYPE_REGULAR;
	    OnIncomingCallRegularType(objUserCallData, objUserData, sDialNo);	
    }
}

void cCallEvent::OnIncomingCallQueueType(cCallData^ objCallData, unsigned int unQueueId)
{
    objCallData->m_unQueueId = unQueueId;
    cServerSIP::m_objServerSIP->AcceptCall(objCallData);
}

void cCallEvent::OnIncomingCallStealthType(cCallData^ objCallData, cUserData^ objUserData, String^ sDialNo)
{
    objCallData->m_sStealthDialNo = sDialNo;
    objCallData->m_sStealthDialNo->ToLower();

    if (cServerSIP::m_objServerSIP->AcceptCall(objCallData))
        objUserData->m_nCallCount++;
}

void cCallEvent::OnIncomingCallRegularType(cCallData^ objCallData, cUserData^ objUserData, String^ sDialNo)
{
    int nDialPlanIndex = cServerSIP::m_objServerSIP->m_objDialPlanList->SearchDialPlan(sDialNo);
	
    if(nDialPlanIndex != -1)
    {
	    String^ sLineName = gcnew String("");
	    sDialNo = cServerSIP::m_objServerSIP->m_objDialPlanList->ApplyDialPlan(nDialPlanIndex, sDialNo, sLineName);

        if (cServerSIP::m_objServerSIP->DialCallToLine(objCallData, sDialNo, sLineName) != nullptr)
            objUserData->m_nCallCount++;
    }
    else
    {
        if (cServerSIP::m_objServerSIP->DialCallToUser(objCallData, sDialNo) != nullptr)
            objUserData->m_nCallCount++;
    }		
}

////////////////////////////////////////////////////

void cCallEvent::OnAcceptIncomingCallConnected(cCallData^ objCallData) 
{
    objCallData->m_bConnected = true;
	
    switch(objCallData->m_nCallType)
    {
        case CALL_TYPE_STEALTH:
		    OnConnectedCallStealthType(objCallData);
		    break;

        case CALL_TYPE_QUEUE:
		    OnConnectedCallQueueType(objCallData);
		    break;

        case CALL_TYPE_REGULAR:
		    break;
    }
}

void cCallEvent::OnDialCallConnected(cCallData^ objCallData) 
{
    objCallData->m_bConnected = true;
							
    switch(objCallData->m_nCallType)
    {
        case CALL_TYPE_REGULAR:
		    OnConnectedCallRegularType(objCallData);
		    break;

        case CALL_TYPE_QUEUE:
		    break;

        case CALL_TYPE_STEALTH:
		    break;
    }
}

////////////////////////////////////////////////////////

void cCallEvent::OnConnectedCallRegularType(cCallData^ objCallData)
{
    cCallData^ objBindCallData = objCallData->m_objBindCallData;

    if (objBindCallData != nullptr && objBindCallData->m_bConnected == false)
    {
        if (cServerSIP::m_objServerSIP->AcceptCall(objBindCallData) == false)
        {
            cServerSIP::m_objServerSIP->CloseCall(objBindCallData);
            cServerSIP::m_objServerSIP->CloseCall(objCallData);

            return;
        }
    }
	
    cServerSIP::m_objServerSIP->StartVoiceStream(objCallData, objBindCallData);
	
    cServerSIP::m_objServerSIP->m_objStealthList->TryCaptureCall(objCallData);
    cServerSIP::m_objServerSIP->m_objStealthList->TryCaptureCall(objBindCallData);
}

void cCallEvent::OnConnectedCallStealthType(cCallData^ objCallData)
{
    cServerSIP::m_objServerSIP->m_objStealthList->AddCallToStealthList(objCallData);

    int nStealthSessionId = cServerSIP::m_objServerSIP->m_objServerData->GetStealthSessionId();
    cServerSIP::m_objServerSIP->AddCallToSession(objCallData, nStealthSessionId);
}

void cCallEvent::OnConnectedCallQueueType(cCallData^ objCallData)
{
    cServerSIP::m_objServerSIP->m_objQueueList->AddCallToQueue(objCallData);
}

////////////////////////////////////////////////////////

void cCallEvent::OnDialCallTimeOut(cCallData^ objCallData) 
{
    cCallData^ objBindCallData = objCallData->m_objBindCallData;

    if(objBindCallData != nullptr)
    {
	    cServerSIP::m_objServerSIP->SendResponse(objBindCallData, 408, "Request Timeout");
	    cServerSIP::m_objServerSIP->CloseCall(objBindCallData);
    }

    cServerSIP::m_objServerSIP->CloseCall(objCallData);
}

void cCallEvent::OnAcceptCallTimeOut(cCallData^ objCallData)
{
    OnDisconnectCall(objCallData);
}

void cCallEvent::OnDisconnectCall(cCallData^ objCallData) 
{
    cCallData^ objBindCallData = objCallData->m_objBindCallData;
	
    if(objBindCallData != nullptr)
	    cServerSIP::m_objServerSIP->CloseCall(objBindCallData);
	
    cServerSIP::m_objServerSIP->CloseCall(objCallData);
}

void cCallEvent::OnIncomingCallCancel(cCallData^ objCallData) 
{
    OnDisconnectCall(objCallData);
}

void cCallEvent::OnSessionLost(cCallData^ objCallData) 
{
    OnDisconnectCall(objCallData);
}