// DialPlanList.cpp: implementation of the CDialPlanList class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ServerSIP.h"
#include "DialPlanList.h"
#include "Registry.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CDialPlanList::CDialPlanList()
{
	m_pDialPlanList = NULL;
	m_nDialPlanCount = 0;
}

CDialPlanList::~CDialPlanList()
{
	UnLoadDialPlans();
}

void CDialPlanList::ReLoadDialPlans()
{
	UnLoadDialPlans();
	LoadDialPlans();
}

void CDialPlanList::UnLoadDialPlans()
{
	delete[] m_pDialPlanList;
	m_pDialPlanList = NULL;

	m_nDialPlanCount = 0;
}

void CDialPlanList::LoadDialPlans()
{
	CStringList objDialPlanList;
	
	CRegistry objReg;
	objReg.EnumDialPlan(&objDialPlanList);
	
	if(objDialPlanList.IsEmpty()) return;

	m_nDialPlanCount = objDialPlanList.GetCount();
	m_pDialPlanList = new CDialPlanData[m_nDialPlanCount];
	
	CDialPlanReg objDialPlanReg;

	for(int nCount = 0; nCount < m_nDialPlanCount; nCount++)
	{
		CString sUserPrefix = objDialPlanList.RemoveHead();
		
		if(!objReg.GetDialPlan(sUserPrefix, &objDialPlanReg))
		{
			CServerSIP::m_pServerSIP->LogError("Unable to extract dial plan information from registry.");
			continue;
		}

		m_pDialPlanList[nCount].m_nUserPrefixSize = sUserPrefix.GetLength();
		m_pDialPlanList[nCount].m_sUserPrefix     = sUserPrefix;
		
		m_pDialPlanList[nCount].m_sLineName       = objDialPlanReg.m_sLineName;
		m_pDialPlanList[nCount].m_sCallPrefix     = objDialPlanReg.m_sCallPrefix;
	}
}

int CDialPlanList::SearchDialPlan(CString sCallTo)
{
	int nCallToLen = sCallTo.GetLength();

	for(int nCount = 0; nCount < m_nDialPlanCount; nCount++)
	{
		int nPrefixLen = m_pDialPlanList[nCount].m_nUserPrefixSize;
		
		if(nCallToLen <= nPrefixLen) continue;
		
		if(sCallTo.Left(nPrefixLen) == m_pDialPlanList[nCount].m_sUserPrefix)
			return nCount;
	}

	return -1;
}

CString CDialPlanList::ApplyDialPlan(int nDialPlanIndex, CString sCallTo, CString& sLineName)
{
	int nPrefixLen = m_pDialPlanList[nDialPlanIndex].m_nUserPrefixSize;
	sCallTo.Delete(0, nPrefixLen);

	sLineName = m_pDialPlanList[nDialPlanIndex].m_sLineName;

	if(m_pDialPlanList[nDialPlanIndex].m_sCallPrefix.IsEmpty())
		return sCallTo;

	return m_pDialPlanList[nDialPlanIndex].m_sCallPrefix + sCallTo;
}

